@@ -375,7 +375,7 @@ gst_lamemp3enc_set_format (GstAudioEncoder * enc, GstAudioInfo * info)
 
   if (tags) {
     gst_audio_encoder_merge_tags (enc, tags, GST_TAG_MERGE_REPLACE);
-    gst_tag_list_free (tags);
+    gst_tag_list_unref (tags);
   }
 
   return TRUE;
@@ -383,7 +383,7 @@ gst_lamemp3enc_set_format (GstAudioEncoder * enc, GstAudioInfo * info)
 zero_output_rate:
   {
     if (tags)
-      gst_tag_list_free (tags);
+      gst_tag_list_unref (tags);
     GST_ELEMENT_ERROR (lame, LIBRARY, SETTINGS, (NULL),
         ("LAME mp3 audio decided on a zero sample rate"));
     return FALSE;
@@ -653,6 +653,7 @@ gst_lamemp3enc_finish_frames (GstLameMP3Enc * lame)
     guint rate, version, layer, size;
     GstBuffer *mp3_buf;
     const guint8 *data;
+    guint samples_per_frame;
 
     data = gst_adapter_map (lame->adapter, 4);
     header = GST_READ_UINT32_BE (data);
@@ -677,11 +678,18 @@ gst_lamemp3enc_finish_frames (GstLameMP3Enc * lame)
       break;
     }
 
+    /* Account for the internal resampling, finish frame really wants to
+     * know about the number of incoming samples
+     */
+    samples_per_frame = (version == 1) ? 1152 : 576;
+    samples_per_frame *= lame->samplerate;
+    samples_per_frame /= lame->out_samplerate;
+
     /* should be ok now */
     mp3_buf = gst_adapter_take_buffer (lame->adapter, size);
     /* number of samples for MPEG-1, layer 3 */
     result = gst_audio_encoder_finish_frame (GST_AUDIO_ENCODER (lame),
-        mp3_buf, version == 1 ? 1152 : 576);
+        mp3_buf, samples_per_frame);
   }
 
 exit:
@@ -819,7 +827,7 @@ gst_lamemp3enc_setup (GstLameMP3Enc * lame, GstTagList ** tags)
   if ((command) < 0) { \
     GST_ERROR_OBJECT (lame, "setup failed: " G_STRINGIFY (command)); \
     if (*tags) { \
-      gst_tag_list_free (*tags); \
+      gst_tag_list_unref (*tags); \
       *tags = NULL; \
     } \
     return FALSE; \
